<?php

namespace Drupal\commerce_begateway\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;

/**
 * Provides the class for payment off-site form.
 *
 * Provide a buildConfigurationForm() method which calls buildRedirectForm()
 * with the right parameters.
 */
class BeGatewayForm extends BasePaymentOffsiteForm {

  /**
   * Gateway plugin.
   *
   * @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface
   */
  private $paymentGatewayPlugin;

  /**
   * Getting plugin's configuration.
   *
   * @param string $configuration
   *   Configuration name.
   *
   * @return mixed
   *   Configuration value.
   */
  private function getConfiguration($configuration) {
    return $this->paymentGatewayPlugin->getConfiguration()[$configuration];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    $this->paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();

    $order        = $payment->getOrder();
    $total_price  = $order->getTotalPrice();
    $data         = array();

    \BeGateway\Settings::$shopId = $this->getConfiguration('shop_id');
    \BeGateway\Settings::$shopKey = $this->getConfiguration('shop_key');
    \BeGateway\Settings::$checkoutBase = 'https://' . $this->getConfiguration('checkout_domain');

    $transaction = new \BeGateway\GetPaymentToken;

    if ($this->getConfiguration('action') == 'authorization') {
      $transaction->setAuthorizationTransactionType();
    }

    $transaction->setTestMode($this->getConfiguration('mode') == 'test');

    $notification_url = $this->paymentGatewayPlugin->getNotifyUrl()->toString();
    $notification_url = str_replace('0.0.0.0', 'webhook.begateway.com:8443', $notification_url);

    $transaction->setNotificationUrl($notification_url);
    $transaction->setSuccessUrl($form['#return_url']);
    $transaction->setDeclineUrl($form['#return_url']);
    $transaction->setFailUrl($form['#return_url']);
    $transaction->setLanguage(\Drupal::languageManager()->getCurrentLanguage()->getId());
    $transaction->setTrackingId($payment->getOrderId());
    $transaction->money->setAmount($total_price ? $total_price->getNumber() : 0);
    $transaction->money->setCurrency($total_price->getCurrencyCode());
    $transaction->setDescription(\Drupal::token()->replace($this->getConfiguration('description'), ['commerce_order' => $order]));

    $timeout = intval($this->getConfiguration('timeout'));

    if ($timeout > 0) {
      $expired_at = date("Y-m-d", ($timeout * 60 + time())) . "T00:00:00+00:00";
      $transaction->setExpiryDate($expired_at);
    }

    try {
      $response = $transaction->submit();
    } catch (Exception $e) { }

    if ($response->isSuccess()) {
      $redirect_url = $response->getRedirectUrlScriptName();
      $data['token'] = $response->getToken();
    } else {
      throw new PaymentGatewayException($response->getMessage());
    }

    return $this->buildRedirectForm($form, $form_state, $redirect_url, $data, 'post');
  }
}
